package lsystemsapp.gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;

import javax.swing.*;
import javax.swing.event.*;

import com.sun.image.codec.jpeg.*;
import lsystemsapp.lsystems.*;

/*
 JPanel na ktorym powstaje obrazek. Umozliwa skalowanie obrazka i
 przesuwanie obrazka za pomoca myszki (LPM skaluje, PPM przesuwa), a
 takze kadrowanie i zapisywanie rysunku na dysku w postaci:
 <gramtyka>_<zestaw_interpretacji_symboli>_<liczba_iteracji>_<kat>.jpg
*/

public class JPanel_DrawPane extends JPanel implements FractalDrawing{
    private Skalowanie s;
    private Grammar grammar;
    private SymbolInterpretationGroup symbolInterpretationGroup;
    private String symbols;
    private TurtleDrawer turtle;
    private int drawing_width=0, drawing_height=0;
    public boolean frame = false;
    private double dimension_x=100, dimension_y=100;
    private double center_x=0, center_y=0;

    public JPanel_DrawPane() {
        MouseInput mouseInput = new MouseInput();
        this.addMouseListener(mouseInput);
        this.addMouseMotionListener(mouseInput);

    }

    public void paintOnGraphics(Graphics2D g){
        turtle = new TurtleDrawer();

        Skalowanie s = new Skalowanie(getWidth(),
                                      getHeight(),
                                      dimension_x, dimension_y,
                                      center_x * dimension_x,
                                      -center_y * dimension_y);
        Graphics2D g2 = (Graphics2D) g;
        g2.setColor(new Color(255, 255, 255));
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);
        g2.fillRect(0, 0, getWidth(), getHeight());
        try{
            turtle.interpretSymbolSequence(symbols, g2, s, grammar, symbolInterpretationGroup);
        }
        catch(NullPointerException e){}
        catch(Exception e){
            e.printStackTrace();
        }

        if(frame) {
            framePicture();
            frame=false;
            this.paintComponent(g);
        }

    }

    public void paintComponent(Graphics g){
        super.paintComponent(g);
        paintOnGraphics((Graphics2D)g);
    }

    public void framePicture(){
        dimension_x = 1.1*(turtle.horizontal_max-turtle.horizontal_min);
        dimension_y = 1.1*(turtle.vertical_max-turtle.vertical_min);
        double sx = (turtle.horizontal_max+turtle.horizontal_min)/2.;
        double sy = (turtle.vertical_max+turtle.vertical_min)/2.;
        //System.out.println("h max/min"+turtle.horizontal_max+", "+turtle.horizontal_min);
        //System.out.println("Sx :"+sx+", Sy :"+sy);
        center_x=-sx/dimension_x;
        center_y=-sy/dimension_y;
    }

    public void saveImageToJPEG(OutputStream out, float quality, int width, int height) throws IOException{
        BufferedImage bi = new BufferedImage(width, height, BufferedImage.TYPE_INT_BGR);
        Graphics2D g2 = bi.createGraphics();
        drawing_width=width;
        drawing_height=height;
        this.paintOnGraphics(g2);
        drawing_width=drawing_height=0;

        JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);
        JPEGEncodeParam params = encoder.getDefaultJPEGEncodeParam(bi);
        params.setQuality(quality, false);
        encoder.setJPEGEncodeParam(params);
        encoder.encode(bi);

    }

    public void saveImageToJPEG(OutputStream out, float quality) throws IOException{
        BufferedImage bi = new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_INT_BGR);
        Graphics2D g2 = bi.createGraphics();
        this.paint(g2);

        JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);
        JPEGEncodeParam params = encoder.getDefaultJPEGEncodeParam(bi);
        params.setQuality(quality, false);
        encoder.setJPEGEncodeParam(params);
        encoder.encode(bi);
    }

    public int getWidth(){
        if(drawing_width==0) return super.getWidth();
        return drawing_width;
    }

    public int getHeight(){
        if(drawing_height==0) return super.getHeight();
        return drawing_height;
    }


    public void setSymbolSequence(String sequence){
        symbols = sequence;
    }

    public void setGrammar(Grammar g){
        grammar = g;
    }

    public void setSymbolInterpretationGroup(SymbolInterpretationGroup g){
        symbolInterpretationGroup=g;
    }

    private void jbInit() throws Exception {
    }

    public class MouseInput extends MouseInputAdapter{

        private double sensitiveness_x=1, sensitiveness_y=1;
        private Point last, next;
        private double last_dimension_y, last_dimension_x;
        private double last_center_y, last_center_x;

        public MouseInput(){
            //updateSensitiveness();
        }

        public void mousePressed(MouseEvent e){
            last = new Point(e.getPoint());
            last_dimension_y = dimension_y;
            last_dimension_x = dimension_x;
            last_center_x = center_x;
            last_center_y = center_y;
            //repaint();
        }

        public void mouseReleased(MouseEvent e){
            //next = new Point(e.getPoint());
        }

        public void mouseDragged(MouseEvent e){
            Point p = e.getPoint();
            if(javax.swing.SwingUtilities.isLeftMouseButton(e)){
                dimension_y = last_dimension_y +
                              (e.getPoint().y - last.y) * sensitiveness_y;
                dimension_x = last_dimension_x +
                              (e.getPoint().x - last.x) * sensitiveness_x;
            }
            if(javax.swing.SwingUtilities.isRightMouseButton(e)){
                center_y = last_center_y -
                              (e.getPoint().y - last.y) * sensitiveness_y/100.;
                center_x = last_center_x +
                              (e.getPoint().x - last.x) * sensitiveness_x/100.;
            }

            repaint();
        }

    }

}
